@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2025 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -19,7 +19,7 @@
   3. This notice may not be removed or altered from any source distribution.
 */
 
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #ifdef SDL_VIDEO_DRIVER_KMSDRM
 
@@ -34,25 +34,23 @@
 
 static SDL_Cursor *KMSDRM_CreateDefaultCursor(void);
 static SDL_Cursor *KMSDRM_CreateCursor(SDL_Surface *surface, int hot_x, int hot_y);
-static int KMSDRM_ShowCursor(SDL_Cursor *cursor);
-static void KMSDRM_MoveCursor(SDL_Cursor *cursor);
+static bool KMSDRM_ShowCursor(SDL_Cursor *cursor);
+static bool KMSDRM_MoveCursor(SDL_Cursor *cursor);
 static void KMSDRM_FreeCursor(SDL_Cursor *cursor);
-static void KMSDRM_WarpMouse(SDL_Window *window, int x, int y);
-static int KMSDRM_WarpMouseGlobal(int x, int y);
 
 /**************************************************************************************/
-/* BEFORE CODING ANYTHING MOUSE/CURSOR RELATED, REMEMBER THIS.                        */
-/* How does SDL manage cursors internally? First, mouse =! cursor. The mouse can have */
-/* many cursors in mouse->cursors.                                                    */
-/* -SDL tells us to create a cursor with KMSDRM_CreateCursor(). It can create many    */
-/*  cursosr with this, not only one.                                                  */
-/* -SDL stores those cursors in a cursors array, in mouse->cursors.                   */
-/* -Whenever it wants (or the programmer wants) takes a cursor from that array        */
-/*  and shows it on screen with KMSDRM_ShowCursor().                                  */
-/*  KMSDRM_ShowCursor() simply shows or hides the cursor it receives: it does NOT     */
-/*  mind if it's mouse->cur_cursor, etc.                                              */
-/* -If KMSDRM_ShowCursor() returns succesfully, that cursor becomes mouse->cur_cursor */
-/*  and mouse->cursor_shown is 1.                                                     */
+// BEFORE CODING ANYTHING MOUSE/CURSOR RELATED, REMEMBER THIS.
+// How does SDL manage cursors internally? First, mouse =! cursor. The mouse can have
+// many cursors in mouse->cursors.
+// -SDL tells us to create a cursor with KMSDRM_CreateCursor(). It can create many
+// cursosr with this, not only one.
+// -SDL stores those cursors in a cursors array, in mouse->cursors.
+// -Whenever it wants (or the programmer wants) takes a cursor from that array
+// and shows it on screen with KMSDRM_ShowCursor().
+// KMSDRM_ShowCursor() simply shows or hides the cursor it receives: it does NOT
+// mind if it's mouse->cur_cursor, etc.
+// -If KMSDRM_ShowCursor() returns successfully, that cursor becomes
+// mouse->cur_cursor and mouse->cursor_shown is 1.
 /**************************************************************************************/
 
 static SDL_Cursor *KMSDRM_CreateDefaultCursor(void)
@@ -60,14 +58,14 @@ static SDL_Cursor *KMSDRM_CreateDefaultCursor(void)
     return SDL_CreateCursor(default_cdata, default_cmask, DEFAULT_CWIDTH, DEFAULT_CHEIGHT, DEFAULT_CHOTX, DEFAULT_CHOTY);
 }
 
-/* Given a display's driverdata, destroy the cursor BO for it.
+/* Given a display's internal, destroy the cursor BO for it.
    To be called from KMSDRM_DestroyWindow(), as that's where we
-   destroy the driverdata for the window's display. */
-void KMSDRM_DestroyCursorBO(_THIS, SDL_VideoDisplay *display)
+   destroy the internal for the window's display. */
+void KMSDRM_DestroyCursorBO(SDL_VideoDevice *_this, SDL_VideoDisplay *display)
 {
-    SDL_DisplayData *dispdata = (SDL_DisplayData *)display->driverdata;
+    SDL_DisplayData *dispdata = display->internal;
 
-    /* Destroy the curso GBM BO. */
+    // Destroy the curso GBM BO.
     if (dispdata->cursor_bo) {
         KMSDRM_gbm_bo_destroy(dispdata->cursor_bo);
         dispdata->cursor_bo = NULL;
@@ -75,34 +73,31 @@ void KMSDRM_DestroyCursorBO(_THIS, SDL_VideoDisplay *display)
     }
 }
 
-/* Given a display's driverdata, create the cursor BO for it.
+/* Given a display's internal, create the cursor BO for it.
    To be called from KMSDRM_CreateWindow(), as that's where we
    build a window and assign a display to it. */
-void KMSDRM_CreateCursorBO(SDL_VideoDisplay *display)
+bool KMSDRM_CreateCursorBO(SDL_VideoDisplay *display)
 {
 
     SDL_VideoDevice *dev = SDL_GetVideoDevice();
-    SDL_VideoData *viddata = ((SDL_VideoData *)dev->driverdata);
-    SDL_DisplayData *dispdata = (SDL_DisplayData *)display->driverdata;
+    SDL_VideoData *viddata = dev->internal;
+    SDL_DisplayData *dispdata = display->internal;
 
     if (!KMSDRM_gbm_device_is_format_supported(viddata->gbm_dev,
                                                GBM_FORMAT_ARGB8888,
                                                GBM_BO_USE_CURSOR | GBM_BO_USE_WRITE)) {
-        SDL_SetError("Unsupported pixel format for cursor");
-        return;
+        return SDL_SetError("Unsupported pixel format for cursor");
     }
 
     if (KMSDRM_drmGetCap(viddata->drm_fd,
                          DRM_CAP_CURSOR_WIDTH, &dispdata->cursor_w) ||
         KMSDRM_drmGetCap(viddata->drm_fd, DRM_CAP_CURSOR_HEIGHT,
                          &dispdata->cursor_h)) {
-        SDL_SetError("Could not get the recommended GBM cursor size");
-        return;
+        return SDL_SetError("Could not get the recommended GBM cursor size");
     }
 
     if (dispdata->cursor_w == 0 || dispdata->cursor_h == 0) {
-        SDL_SetError("Could not get an usable GBM cursor size");
-        return;
+        return SDL_SetError("Could not get an usable GBM cursor size");
     }
 
     dispdata->cursor_bo = KMSDRM_gbm_bo_create(viddata->gbm_dev,
@@ -110,38 +105,36 @@ void KMSDRM_CreateCursorBO(SDL_VideoDisplay *display)
                                                GBM_FORMAT_ARGB8888, GBM_BO_USE_CURSOR | GBM_BO_USE_WRITE | GBM_BO_USE_LINEAR);
 
     if (!dispdata->cursor_bo) {
-        SDL_SetError("Could not create GBM cursor BO");
-        return;
+        return SDL_SetError("Could not create GBM cursor BO");
     }
 
     dispdata->cursor_bo_drm_fd = viddata->drm_fd;
+    return true;
 }
 
-/* Remove a cursor buffer from a display's DRM cursor BO. */
-static int KMSDRM_RemoveCursorFromBO(SDL_VideoDisplay *display)
+// Remove a cursor buffer from a display's DRM cursor BO.
+static bool KMSDRM_RemoveCursorFromBO(SDL_VideoDisplay *display)
 {
-    int ret = 0;
-    SDL_DisplayData *dispdata = (SDL_DisplayData *)display->driverdata;
+    bool result = true;
+
+    SDL_DisplayData *dispdata = display->internal;
     SDL_VideoDevice *video_device = SDL_GetVideoDevice();
-    SDL_VideoData *viddata = ((SDL_VideoData *)video_device->driverdata);
+    SDL_VideoData *viddata = video_device->internal;
 
-    ret = KMSDRM_drmModeSetCursor(viddata->drm_fd,
-                                  dispdata->crtc->crtc_id, 0, 0, 0);
-
-    if (ret) {
-        ret = SDL_SetError("Could not hide current cursor with drmModeSetCursor().");
+    const int rc = KMSDRM_drmModeSetCursor(viddata->drm_fd, dispdata->crtc->crtc_id, 0, 0, 0);
+    if (rc < 0) {
+        result = SDL_SetError("drmModeSetCursor() failed: %s", strerror(-rc));
     }
-
-    return ret;
+    return result;
 }
 
-/* Dump a cursor buffer to a display's DRM cursor BO.  */
-static int KMSDRM_DumpCursorToBO(SDL_VideoDisplay *display, SDL_Cursor *cursor)
+// Dump a cursor buffer to a display's DRM cursor BO.
+static bool KMSDRM_DumpCursorToBO(SDL_VideoDisplay *display, SDL_Cursor *cursor)
 {
-    SDL_DisplayData *dispdata = (SDL_DisplayData *)display->driverdata;
-    KMSDRM_CursorData *curdata = (KMSDRM_CursorData *)cursor->driverdata;
+    SDL_DisplayData *dispdata = display->internal;
+    SDL_CursorData *curdata = cursor->internal;
     SDL_VideoDevice *video_device = SDL_GetVideoDevice();
-    SDL_VideoData *viddata = ((SDL_VideoData *)video_device->driverdata);
+    SDL_VideoData *viddata = video_device->internal;
 
     uint32_t bo_handle;
     size_t bo_stride;
@@ -149,8 +142,8 @@ static int KMSDRM_DumpCursorToBO(SDL_VideoDisplay *display, SDL_Cursor *cursor)
     uint8_t *ready_buffer = NULL;
     uint8_t *src_row;
 
-    int i;
-    int ret;
+    int i, rc;
+    bool result = true;
 
     if (!curdata || !dispdata->cursor_bo) {
         return SDL_SetError("Cursor or display not initialized properly.");
@@ -164,39 +157,33 @@ static int KMSDRM_DumpCursorToBO(SDL_VideoDisplay *display, SDL_Cursor *cursor)
     ready_buffer = (uint8_t *)SDL_calloc(1, bufsize);
 
     if (!ready_buffer) {
-        ret = SDL_OutOfMemory();
+        result = false;
         goto cleanup;
     }
 
-    /* Copy from the cursor buffer to a buffer that we can dump to the GBM BO. */
+    // Copy from the cursor buffer to a buffer that we can dump to the GBM BO.
     for (i = 0; i < curdata->h; i++) {
         src_row = &((uint8_t *)curdata->buffer)[i * curdata->w * 4];
         SDL_memcpy(ready_buffer + (i * bo_stride), src_row, (size_t)4 * curdata->w);
     }
 
-    /* Dump the cursor buffer to our GBM BO. */
+    // Dump the cursor buffer to our GBM BO.
     if (KMSDRM_gbm_bo_write(dispdata->cursor_bo, ready_buffer, bufsize)) {
-        ret = SDL_SetError("Could not write to GBM cursor BO");
+        result = SDL_SetError("Could not write to GBM cursor BO");
         goto cleanup;
     }
 
-    /* Put the GBM BO buffer on screen using the DRM interface. */
+    // Put the GBM BO buffer on screen using the DRM interface.
     bo_handle = KMSDRM_gbm_bo_get_handle(dispdata->cursor_bo).u32;
     if (curdata->hot_x == 0 && curdata->hot_y == 0) {
-        ret = KMSDRM_drmModeSetCursor(viddata->drm_fd, dispdata->crtc->crtc_id,
+        rc = KMSDRM_drmModeSetCursor(viddata->drm_fd, dispdata->crtc->crtc_id,
                                       bo_handle, dispdata->cursor_w, dispdata->cursor_h);
     } else {
-        ret = KMSDRM_drmModeSetCursor2(viddata->drm_fd, dispdata->crtc->crtc_id,
+        rc = KMSDRM_drmModeSetCursor2(viddata->drm_fd, dispdata->crtc->crtc_id,
                                        bo_handle, dispdata->cursor_w, dispdata->cursor_h, curdata->hot_x, curdata->hot_y);
     }
-
-    if (ret) {
-        ret = SDL_SetError("Failed to set DRM cursor.");
-        goto cleanup;
-    }
-
-    if (ret) {
-        ret = SDL_SetError("Failed to reset cursor position.");
+    if (rc < 0) {
+        result = SDL_SetError("Failed to set DRM cursor: %s", strerror(-rc));
         goto cleanup;
     }
 
@@ -205,25 +192,25 @@ cleanup:
     if (ready_buffer) {
         SDL_free(ready_buffer);
     }
-    return ret;
+    return result;
 }
 
-/* This is only for freeing the SDL_cursor.*/
+// This is only for freeing the SDL_cursor.
 static void KMSDRM_FreeCursor(SDL_Cursor *cursor)
 {
-    KMSDRM_CursorData *curdata;
+    SDL_CursorData *curdata;
 
-    /* Even if the cursor is not ours, free it. */
+    // Even if the cursor is not ours, free it.
     if (cursor) {
-        curdata = (KMSDRM_CursorData *)cursor->driverdata;
-        /* Free cursor buffer */
+        curdata = cursor->internal;
+        // Free cursor buffer
         if (curdata->buffer) {
             SDL_free(curdata->buffer);
             curdata->buffer = NULL;
         }
-        /* Free cursor itself */
-        if (cursor->driverdata) {
-            SDL_free(cursor->driverdata);
+        // Free cursor itself
+        if (cursor->internal) {
+            SDL_free(cursor->internal);
         }
         SDL_free(cursor);
     }
@@ -234,24 +221,22 @@ static void KMSDRM_FreeCursor(SDL_Cursor *cursor)
    in dispata) is destroyed and recreated when we recreate windows, etc. */
 static SDL_Cursor *KMSDRM_CreateCursor(SDL_Surface *surface, int hot_x, int hot_y)
 {
-    KMSDRM_CursorData *curdata;
-    SDL_Cursor *cursor, *ret;
+    SDL_CursorData *curdata;
+    SDL_Cursor *cursor, *result;
 
     curdata = NULL;
-    ret = NULL;
+    result = NULL;
 
     cursor = (SDL_Cursor *)SDL_calloc(1, sizeof(*cursor));
     if (!cursor) {
-        SDL_OutOfMemory();
         goto cleanup;
     }
-    curdata = (KMSDRM_CursorData *)SDL_calloc(1, sizeof(*curdata));
+    curdata = (SDL_CursorData *)SDL_calloc(1, sizeof(*curdata));
     if (!curdata) {
-        SDL_OutOfMemory();
         goto cleanup;
     }
 
-    /* hox_x and hot_y are the coordinates of the "tip of the cursor" from it's base. */
+    // hox_x and hot_y are the coordinates of the "tip of the cursor" from it's base.
     curdata->hot_x = hot_x;
     curdata->hot_y = hot_y;
     curdata->w = surface->w;
@@ -265,7 +250,6 @@ static SDL_Cursor *KMSDRM_CreateCursor(SDL_Surface *surface, int hot_x, int hot_
     curdata->buffer = (uint32_t *)SDL_malloc(curdata->buffer_size);
 
     if (!curdata->buffer) {
-        SDL_OutOfMemory();
         goto cleanup;
     }
 
@@ -274,15 +258,15 @@ static SDL_Cursor *KMSDRM_CreateCursor(SDL_Surface *surface, int hot_x, int hot_
        alpha-premultiplied, but the SDL surface we receive has
        straight-alpha pixels, so we always have to convert. */
     SDL_PremultiplyAlpha(surface->w, surface->h,
-                         surface->format->format, surface->pixels, surface->pitch,
-                         SDL_PIXELFORMAT_ARGB8888, curdata->buffer, surface->w * 4);
+                         surface->format, surface->pixels, surface->pitch,
+                         SDL_PIXELFORMAT_ARGB8888, curdata->buffer, surface->w * 4, true);
 
-    cursor->driverdata = curdata;
+    cursor->internal = curdata;
 
-    ret = cursor;
+    result = cursor;
 
 cleanup:
-    if (!ret) {
+    if (!result) {
         if (curdata) {
             if (curdata->buffer) {
                 SDL_free(curdata->buffer);
@@ -294,111 +278,91 @@ cleanup:
         }
     }
 
-    return ret;
+    return result;
 }
 
-/* Show the specified cursor, or hide if cursor is NULL or has no focus. */
-static int KMSDRM_ShowCursor(SDL_Cursor *cursor)
+// Show the specified cursor, or hide if cursor is NULL or has no focus.
+static bool KMSDRM_ShowCursor(SDL_Cursor *cursor)
 {
     SDL_VideoDisplay *display;
     SDL_Window *window;
-    SDL_Mouse *mouse;
+    SDL_Mouse *mouse = SDL_GetMouse();
 
-    int num_displays, i;
-    int ret = 0;
-
-    /* Get the mouse focused window, if any. */
-
-    mouse = SDL_GetMouse();
-    if (!mouse) {
-        return SDL_SetError("No mouse.");
-    }
+    int i;
+    bool result = true;
 
+    // Get the mouse focused window, if any.
     window = mouse->focus;
 
     if (!window || !cursor) {
-
         /* If no window is focused by mouse or cursor is NULL,
            since we have no window (no mouse->focus) and hence
            we have no display, we simply hide mouse on all displays.
            This happens on video quit, where we get here after
            the mouse focus has been unset, yet SDL wants to
            restore the system default cursor (makes no sense here). */
-
-        num_displays = SDL_GetNumVideoDisplays();
-
-        /* Iterate on the displays hidding the cursor. */
-        for (i = 0; i < num_displays; i++) {
-            display = SDL_GetDisplay(i);
-            ret = KMSDRM_RemoveCursorFromBO(display);
+        SDL_DisplayID *displays = SDL_GetDisplays(NULL);
+        if (displays) {
+            // Iterate on the displays, hiding the cursor.
+            for (i = 0; i < displays[i]; i++) {
+                display = SDL_GetVideoDisplay(displays[i]);
+                result = KMSDRM_RemoveCursorFromBO(display);
+            }
+            SDL_free(displays);
         }
-
     } else {
-
-        display = SDL_GetDisplayForWindow(window);
-
+        display = SDL_GetVideoDisplayForWindow(window);
         if (display) {
-
             if (cursor) {
                 /* Dump the cursor to the display DRM cursor BO so it becomes visible
                    on that display. */
-                ret = KMSDRM_DumpCursorToBO(display, cursor);
-
+                result = KMSDRM_DumpCursorToBO(display, cursor);
             } else {
-                /* Hide the cursor on that display. */
-                ret = KMSDRM_RemoveCursorFromBO(display);
+                // Hide the cursor on that display.
+                result = KMSDRM_RemoveCursorFromBO(display);
             }
         }
     }
 
-    return ret;
+    return result;
 }
 
-/* Warp the mouse to (x,y) */
-static void KMSDRM_WarpMouse(SDL_Window *window, int x, int y)
-{
-    /* Only one global/fullscreen window is supported */
-    KMSDRM_WarpMouseGlobal(x, y);
-}
-
-/* Warp the mouse to (x,y) */
-static int KMSDRM_WarpMouseGlobal(int x, int y)
+static bool KMSDRM_WarpMouseGlobal(float x, float y)
 {
     SDL_Mouse *mouse = SDL_GetMouse();
 
     if (mouse && mouse->cur_cursor && mouse->focus) {
-
         SDL_Window *window = mouse->focus;
-        SDL_DisplayData *dispdata = (SDL_DisplayData *)SDL_GetDisplayForWindow(window)->driverdata;
+        SDL_DisplayData *dispdata = SDL_GetDisplayDriverDataForWindow(window);
 
-        /* Update internal mouse position. */
-        SDL_SendMouseMotion(mouse->focus, mouse->mouseID, 0, x, y);
+        // Update internal mouse position.
+        SDL_SendMouseMotion(0, mouse->focus, SDL_GLOBAL_MOUSE_ID, false, x, y);
 
-        /* And now update the cursor graphic position on screen. */
+        // And now update the cursor graphic position on screen.
         if (dispdata->cursor_bo) {
-            int ret = 0;
-
-            ret = KMSDRM_drmModeMoveCursor(dispdata->cursor_bo_drm_fd, dispdata->crtc->crtc_id, x, y);
-
-            if (ret) {
-                SDL_SetError("drmModeMoveCursor() failed.");
+            const int rc = KMSDRM_drmModeMoveCursor(dispdata->cursor_bo_drm_fd, dispdata->crtc->crtc_id, (int)x, (int)y);
+            if (rc < 0) {
+                return SDL_SetError("drmModeMoveCursor() failed: %s", strerror(-rc));
             }
-
-            return ret;
-
+            return true;
         } else {
             return SDL_SetError("Cursor not initialized properly.");
         }
-
     } else {
         return SDL_SetError("No mouse or current cursor.");
     }
 }
 
-void KMSDRM_InitMouse(_THIS, SDL_VideoDisplay *display)
+static bool KMSDRM_WarpMouse(SDL_Window *window, float x, float y)
+{
+    // Only one global/fullscreen window is supported
+    return KMSDRM_WarpMouseGlobal(x, y);
+}
+
+void KMSDRM_InitMouse(SDL_VideoDevice *_this, SDL_VideoDisplay *display)
 {
     SDL_Mouse *mouse = SDL_GetMouse();
-    SDL_DisplayData *dispdata = (SDL_DisplayData *)display->driverdata;
+    SDL_DisplayData *dispdata = display->internal;
 
     mouse->CreateCursor = KMSDRM_CreateCursor;
     mouse->ShowCursor = KMSDRM_ShowCursor;
@@ -411,41 +375,36 @@ void KMSDRM_InitMouse(_THIS, SDL_VideoDisplay *display)
        we don't want several cursors to be created for the same display. */
     if (!dispdata->default_cursor_init) {
         SDL_SetDefaultCursor(KMSDRM_CreateDefaultCursor());
-        dispdata->default_cursor_init = SDL_TRUE;
+        dispdata->default_cursor_init = true;
     }
 }
 
-void KMSDRM_QuitMouse(_THIS)
+void KMSDRM_QuitMouse(SDL_VideoDevice *_this)
 {
-    /* TODO: ? */
+    // TODO: ?
 }
 
-/* This is called when a mouse motion event occurs */
-static void KMSDRM_MoveCursor(SDL_Cursor *cursor)
+// This is called when a mouse motion event occurs
+static bool KMSDRM_MoveCursor(SDL_Cursor *cursor)
 {
     SDL_Mouse *mouse = SDL_GetMouse();
-    int ret = 0;
 
     /* We must NOT call SDL_SendMouseMotion() here or we will enter recursivity!
        That's why we move the cursor graphic ONLY. */
     if (mouse && mouse->cur_cursor && mouse->focus) {
-
         SDL_Window *window = mouse->focus;
-        SDL_DisplayData *dispdata = (SDL_DisplayData *)SDL_GetDisplayForWindow(window)->driverdata;
+        SDL_DisplayData *dispdata = SDL_GetDisplayDriverDataForWindow(window);
 
         if (!dispdata->cursor_bo) {
-            SDL_SetError("Cursor not initialized properly.");
-            return;
+            return SDL_SetError("Cursor not initialized properly.");
         }
 
-        ret = KMSDRM_drmModeMoveCursor(dispdata->cursor_bo_drm_fd, dispdata->crtc->crtc_id, mouse->x, mouse->y);
-
-        if (ret) {
-            SDL_SetError("drmModeMoveCursor() failed.");
+        const int rc = KMSDRM_drmModeMoveCursor(dispdata->cursor_bo_drm_fd, dispdata->crtc->crtc_id, (int)mouse->x, (int)mouse->y);
+        if (rc < 0) {
+            return SDL_SetError("drmModeMoveCursor() failed: %s", strerror(-rc));
         }
     }
+    return true;
 }
 
-#endif /* SDL_VIDEO_DRIVER_KMSDRM */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_VIDEO_DRIVER_KMSDRM
