@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2025 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -24,10 +24,8 @@
 
 #include "SDL_vitavideo.h"
 
-extern SDL_GLContext VITA_GL_CreateContext(_THIS, SDL_Window *window);
-extern int VITA_GL_LoadLibrary(_THIS, const char *path);
-extern void *VITA_GL_GetProcAddress(_THIS, const char *proc);
+extern SDL_GLContext VITA_GL_CreateContext(SDL_VideoDevice *_this, SDL_Window *window);
+extern int VITA_GL_LoadLibrary(SDL_VideoDevice *_this, const char *path);
+extern SDL_FunctionPointer VITA_GL_GetProcAddress(SDL_VideoDevice *_this, const char *proc);
 
-#endif /* SDL_vitagl_pvr_c_h_ */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_vitagl_pvr_c_h_
